package com.miya.draw.controller;

import cn.hutool.core.bean.copier.BeanCopier;
import cn.hutool.core.bean.copier.CopyOptions;
import com.alibaba.fastjson.JSON;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.miya.draw.config.response.RestResult;
import com.miya.draw.config.response.RestResultUtil;
import com.miya.draw.manager.IDrawManager;
import com.miya.draw.model.DrawModel;
import com.miya.draw.request.*;
import com.miya.draw.response.DrawClickResponse;
import com.miya.draw.response.DrawUserInfoGainResponse;
import com.miya.draw.response.MinePrizeGainResponse;
import com.miya.draw.response.PrizeDrawListResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 抽奖Controller
 *
 * @author jack
 */
@Api(tags = "抽奖模块")
@RestController
@RequestMapping(value = "/draw/")
@Slf4j
public class DrawController {

    @Autowired
    private IDrawManager drawManager;

    @ApiOperationSupport(author = "jack")
    @ApiOperation(value = "用户点击活动页面弹出的信息提示")
    @PostMapping(value = "user/info/gain")
    public RestResult<DrawUserInfoGainResponse> gainUserInfo(@RequestBody @Valid DrawUserInfoGainRequest request) {
        log.info("request="+JSON.toJSONString(request));
        DrawModel drawModel = drawManager.gainActivityUserInfo(request);
        log.info(JSON.toJSONString(drawModel));
        BeanCopier<DrawUserInfoGainResponse> copier = BeanCopier.create(drawModel, DrawUserInfoGainResponse.builder().build(), CopyOptions.create());
        return RestResultUtil.createSuccessResult(copier.copy());
    }

    @ApiOperationSupport(author = "jack")
    @ApiOperation(value = "用户点击活抽奖")
    @PostMapping(value = "draw/click")
    public RestResult<DrawClickResponse> drawClick(@RequestBody @Valid DrawClickRequest request) throws InterruptedException {
        DrawModel drawModel = drawManager.drawClick(request.getPhone());
        BeanCopier<DrawClickResponse> beanCopier = BeanCopier.create(drawModel, DrawClickResponse.builder().build(), CopyOptions.create());
        DrawClickResponse response = beanCopier.copy();
        return RestResultUtil.createSuccessResult(response);
    }

    @ApiOperationSupport(author = "jack")
    @ApiOperation(value = "获得我的奖品")
    @PostMapping(value = "gain/mine/prize")
    public RestResult<List<MinePrizeGainResponse>> gainMinePrize(@RequestBody @Valid MinePrizeGainRequest request) {
        List<DrawModel> drawModelList = drawManager.listMinePrizeList(request.getPhone());
        List<MinePrizeGainResponse> minePrizeGainResponseList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(drawModelList)) {
            minePrizeGainResponseList = drawModelList.stream().map(drawModel -> {
                BeanCopier<MinePrizeGainResponse> copier = BeanCopier.create(drawModel, MinePrizeGainResponse.builder().build(), CopyOptions.create());
                MinePrizeGainResponse response = copier.copy();
                response.setMyPrizeId(drawModel.getId());
                return response;
            }).collect(Collectors.toList());
        }
        return RestResultUtil.createSuccessResult(minePrizeGainResponseList);
    }

    @ApiOperationSupport(author = "jack")
    @ApiOperation(value = "用户填写地址")
    @PostMapping(value = "prize/address/modify")
    public RestResult<Void> modifyPrizeAddress(@RequestBody @Valid PrizeAddressModifyRequest request) {
        drawManager.updateMinePrizeAddress(request.getPhone(), request.getMyPrizeId(), request.getConsigneeName(),
                request.getConsigneePhone(), request.getConsigneeAddress(), request.getConsigneeIdcard());
        return RestResultUtil.createSuccessResult();
    }

    @ApiOperationSupport(author = "jack")
    @ApiOperation(value = "整体中奖情况")
    @PostMapping(value = "prize/draw/list")
    public RestResult<List<PrizeDrawListResponse>> prizeDrawList(@RequestBody @Valid PrizeDrawListRequest request) {
        List<DrawModel> drawModelList = drawManager.listAllWinningInfo(request.getSize());
        drawModelList.forEach(x->
        {
            if(StringUtils.isNotBlank(x.getAccountName())){
                x.setAccountName(x.getAccountName().replaceAll("(\\d{3})\\d{4}(\\d{4})", "$1****$2"));
            }
        });
        List<PrizeDrawListResponse> prizeDrawListResponses = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(drawModelList)) {
            prizeDrawListResponses = drawModelList.stream().map(drawModel -> {
                BeanCopier<PrizeDrawListResponse> copier = BeanCopier.create(drawModel, PrizeDrawListResponse.builder().build(), CopyOptions.create());
                return copier.copy();
            }).collect(Collectors.toList());
        }
        return RestResultUtil.createSuccessResult(prizeDrawListResponses);
    }

}
